#include <vcl.h>
#pragma hdrstop

// vcl
#include <tchar.h>

// self
#include "formMain.h"

// vcl-stub
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;

// constants
namespace
{
   unsigned leftAxisValues_SI[] = { 10, 40, 50, 70, 100, 150, 200, 300, 400, 500, 700, 1000, 1500, 2000, 3000, 4000 };
   double bottomAxisValues_SI[] = { 1, 1.5, 2, 3, 4, 6, 8, 10, 15, 20, 25 };

   const unsigned DEFAULT_LINE_SERIES_PEN_WIDTH = 2;
   const unsigned FOCUSED_LINE_SERIES_PEN_WIDTH = 4;

   const double MIN_ALLOWED_DP = 1.5;
}

// aux.
template <typename T, int size> unsigned ArraySize(const T (& /* refToArray */)[size])
{
   return size;
}

double InterpolateValueLinear(double x1, double y1, double x2, double y2, double x)
{
   double dy = y2 - y1;
   double dx = x2 - x1;
   double d = (dy / dx) * (x - x1);

   double result = y1 + d;

   return result;
}

// ----------------------------------------------------------------------------
// Definition
// ----------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner) :
   TForm(Owner)
{
   PrepareDiagram();
   DrawDiagram();
}

void TForm1::Initialize(void)
{
   chart_->OnBeforeDrawSeries = OnBeforeDrawSeries;

   Clear();
}

// functional
void TForm1::Clear(void)
{
   chart_->RemoveAllSeries();
}

void TForm1::Redraw(void)
{
   chart_->Repaint();
}

// functional
void TForm1::PrepareDiagram(void)
{
   // prepare axis
   chart_->LeftAxis->Logarithmic = true;
   chart_->LeftAxis->Items->Clear();
   chart_->LeftAxis->Items->Automatic = false;
   chart_->LeftAxis->Title->Caption = _T("Q0");
   chart_->LeftAxis->Grid->SmallDots = true;

   chart_->BottomAxis->Logarithmic = true;
   chart_->BottomAxis->Items->Clear();
   chart_->BottomAxis->Items->Automatic = false;
   chart_->BottomAxis->Title->Caption = _T("dp");
   chart_->BottomAxis->Grid->SmallDots = true;

   // setup chart
   unsigned labelCountLeftAxis = ArraySize(leftAxisValues_SI);
   unsigned* labelsLeftAxis = &leftAxisValues_SI[0];
   unsigned labelCountBottomAxis = ArraySize(bottomAxisValues_SI);
   double* labelsBottomAxis = &bottomAxisValues_SI[0];

   chart_->LeftAxis->Maximum = labelsLeftAxis[labelCountLeftAxis - 1];
   chart_->LeftAxis->Minimum = labelsLeftAxis[0];
   chart_->BottomAxis->Maximum = labelsBottomAxis[labelCountBottomAxis - 1];
   chart_->BottomAxis->Minimum = labelsBottomAxis[0];

   // draw-axes
   for(unsigned i = 0; i < labelCountLeftAxis; ++i)
      chart_->LeftAxis->Items->Add(labelsLeftAxis[i], String(labelsLeftAxis[i]));

   for(unsigned j = 0; j < labelCountBottomAxis; ++j)
      chart_->BottomAxis->Items->Add(labelsBottomAxis[j], String(labelsBottomAxis[j]));
}

void TForm1::DrawDiagram(void)
{
   Clear();

   TLineSeries* const series = new TLineSeries(chart_);
   series->Color = clYellow;
   series->Marks->Style = smsSeriesTitle;
   series->Marks->Visible = false;
   series->Marks->AutoPosition = true;
   series->LinePen->Style = psDot;
   series->LinePen->Width = DEFAULT_LINE_SERIES_PEN_WIDTH;
   series->ClickableLine = false;
   series->ColorEachLine = false;
   series->ColorEachPoint = false;
   series->Pointer->Style = psNothing;
   series->Pointer->HorizSize = 2;
   series->Pointer->VertSize = 2;
   series->Pointer->Visible = true;
   series->OnGetPointerStyle = OnGetPointerStyle;

   GenerateSeriesValues(*series);

   chart_->AddSeries(series);
}

// access
void TForm1::GenerateSeriesValues(TCustomSeries& series)
{
   double dp = 0.5;
   double q0 = 50;
   double lastBeforeThresholdX = 0.0, lastBeforeThresholdY = 9999;
   double firstAfterThresholdX = 99.9, firstAfterThresholdY = 9999;

   // constants
   for(double t0 = 0; t0 < 60; ++t0)
   {
      dp = dp + 0.1;
      q0 = q0 + 100;

      if( (dp < MIN_ALLOWED_DP) && (dp > lastBeforeThresholdX) )
      {
         lastBeforeThresholdX = dp;
         lastBeforeThresholdY = q0;
      }
      else if((dp > MIN_ALLOWED_DP) && (dp < firstAfterThresholdX) )
      {
         firstAfterThresholdX = dp;
         firstAfterThresholdY = q0;
      }

      series.AddXY(dp, q0);
   }

   // add fixed point add 1.5bar
   double dp_t1 = MIN_ALLOWED_DP;
   double q0_at_threshold = InterpolateValueLinear(lastBeforeThresholdX, lastBeforeThresholdY, firstAfterThresholdX, firstAfterThresholdY, dp_t1);
   series.AddXY(dp_t1, q0_at_threshold);

   double dp_t2 = MIN_ALLOWED_DP + 0.01;
   double q0_at_threshold2 = InterpolateValueLinear(lastBeforeThresholdX, lastBeforeThresholdY, firstAfterThresholdX, firstAfterThresholdY, dp_t2);
   series.AddXY(dp_t2, q0_at_threshold2);
}

// events
void __fastcall TForm1::OnBeforeDrawSeries(TObject* sender)
{
   if(TLineSeries* const lineSeries = dynamic_cast<TLineSeries*>(sender))
   {
      lineSeries->LinePen->Style = psDot;
   }
}

Teengine::TSeriesPointerStyle __fastcall TForm1::OnGetPointerStyle(Teengine::TChartSeries* sender, int valueIndex)
{
   TSeriesPointerStyle pointerStyle = TSeriesPointerStyle::psNothing;

   if(TLineSeries* const lineSeries = dynamic_cast<TLineSeries*>(sender))
   {
      lineSeries->LinePen->Style = psDot;

      if(valueIndex > 0)
      {
         double n = lineSeries->XValues->Items[valueIndex];
         if(n >= MIN_ALLOWED_DP)
         {
            lineSeries->LinePen->Style = psSolid;
         }
      }
   }

   return pointerStyle;
}

